"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jest_snapshot_1 = require("jest-snapshot");
const object_hash_1 = __importDefault(require("object-hash"));
const cacheMap = new Map();
const keyMap = new Map();
exports.default = ({ cachePath, updateSnapshot }) => {
    if (!cacheMap.has(cachePath)) {
        cacheMap.set(cachePath, new jest_snapshot_1.SnapshotState(cachePath, { updateSnapshot: updateSnapshot }));
    }
    const snapshotState = cacheMap.get(cachePath);
    const markAsDirty = (key) => {
        snapshotState._dirty = true;
        delete snapshotState._snapshotData[key];
    };
    const match = (page) => {
        const testName = page.name;
        const currentHash = object_hash_1.default(page);
        const { added, updated } = snapshotState;
        // run snapshot to see if there was a match
        const { pass, key } = snapshotState.match({ testName, received: currentHash });
        keyMap.set(testName, key);
        // visual regression should be run on new pages and on pages that have uncommitted changes
        const match = pass && added === snapshotState.added && updated === snapshotState.updated;
        return !match;
    };
    return {
        getChangedPages(pages) {
            return pages.filter(match);
        },
        markAsDirty(page) {
            const key = keyMap.get(page.name);
            if (key)
                markAsDirty(key);
        },
        commitCache() {
            snapshotState.save();
            cacheMap.delete(cachePath);
            keyMap.clear();
        },
    };
};
//# sourceMappingURL=change-tracker.js.map